<?php

/**
 * JCH Optimize - Performs several front-end optimizations for fast downloads.
 *
 * @author    Samuel Marshall <samuel@jch-optimize.net>
 * @copyright Copyright (c) 2020 Samuel Marshall / JCH Optimize
 * @license   GNU/GPLv3, or later. See LICENSE file
 *
 * If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */

namespace JchOptimize\Model;

use _JchOptimizeVendor\Joomla\DI\ContainerAwareInterface;
use _JchOptimizeVendor\Joomla\DI\ContainerAwareTrait;
use _JchOptimizeVendor\Joomla\Model\DatabaseModelInterface;
use _JchOptimizeVendor\Joomla\Model\DatabaseModelTrait;
use _JchOptimizeVendor\Joomla\Model\StatefulModelInterface;
use _JchOptimizeVendor\Joomla\Model\StatefulModelTrait;
use JchOptimize\Core\Admin\Ajax\Ajax as AdminAjax;
use JchOptimize\Core\Admin\Icons;
use JchOptimize\Core\Exception\ExceptionInterface;
use JchOptimize\Core\PageCache\CaptureCache;
use Joomla\Registry\Registry;

\defined('_JEXEC') or exit('Restricted Access');
class Configure implements DatabaseModelInterface, StatefulModelInterface, ContainerAwareInterface
{
    use DatabaseModelTrait;
    use StatefulModelTrait;
    use \JchOptimize\Model\SaveSettingsTrait;
    use ContainerAwareTrait;
    protected string $name = 'configure';

    /**
     * @var ModeSwitcher
     */
    private \JchOptimize\Model\ModeSwitcher $modeSwitcherModel;

    public function __construct(Registry $params, ModeSwitcher $modeSwitcherModel)
    {
        $this->modeSwitcherModel = $modeSwitcherModel;
        $this->setState($params);
    }

    /**
     * @throws ExceptionInterface
     */
    public function applyAutoSettings($autoSetting)
    {
        $aAutoParams = Icons::autoSettingsArrayMap();
        $aSelectedSetting = \array_column($aAutoParams, $autoSetting);
        $aSettingsToApply = \array_combine(\array_keys($aAutoParams), $aSelectedSetting);
        foreach ($aSettingsToApply as $setting => $value) {
            $this->state->set($setting, $value);
        }
        $this->state->set('combine_files_enable', '1');
        $this->saveSettings();
    }

    /**
     * @param mixed $setting
     *
     * @throws ExceptionInterface
     */
    public function toggleSetting($setting): bool
    {
        if (\is_null($setting)) {
            // @TODO some logging here
            return \false;
        }
        if ('integrated_page_cache_enable' == $setting) {
            try {
                $this->modeSwitcherModel->togglePageCacheState();
                if (JCH_PRO) {
                    // @see CaptureCache::updateHtaccess()
                    $this->container->get(CaptureCache::class)->updateHtaccess();
                }

                return \true;
            } catch (\Exception $e) {
                return \false;
            }
        }
        $iCurrentSetting = (int) $this->state->get($setting);
        $newSetting = (string) \abs($iCurrentSetting - 1);
        if ('pro_reduce_unused_css' == $setting && '1' == $newSetting) {
            $this->state->set('optimizeCssDelivery_enable', '1');
        }
        if ('optimizeCssDelivery_enable' == $setting && '0' == $newSetting) {
            $this->state->set('pro_reduce_unused_css', '0');
        }
        if ('pro_smart_combine' == $setting) {
            if ('1' == $newSetting) {
                $aSCValues = AdminAjax::getInstance('SmartCombine')->run();
                $aValues = \array_merge($aSCValues->data['css'], $aSCValues->data['js']);
                $this->state->set('pro_smart_combine_values', \rawurlencode(\json_encode($aValues)));
            } else {
                $this->state->set('pro_smart_combine_values', '');
            }
        }
        $this->state->set($setting, $newSetting);
        $this->saveSettings();

        return \true;
    }
}
